@@ -36,9 +36,14 @@
 #define ENVRETRY	3
 #define VZCTLDEV	"/dev/vzctl"
 
-static _syscall1(long, setluid, uid_t, uid);
+
 static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode);
 
+static inline int setluid(uid_t uid)
+{
+	return syscall(__NR_setluid, uid);
+}
+
 int vz_env_create_data_ioctl(vps_handler *h,
 	struct vzctl_env_create_data *data)
 {
@@ -637,7 +642,7 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 		return VZ_VE_ROOT_NOTSET;
 	if (!vps_is_run(h, veid)) {
 		logger(0, 0, "Unable to stop: VPS is not running");
-		return VZ_VE_NOT_RUNNING;
+		return 0;
 	}
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
 		snprintf(buf, sizeof(buf), VPS_CONF_DIR "%d.%s", veid,
