@@ -31,9 +31,10 @@
 #include "logger.h"
 #include "script.h"
 
-static volatile sig_atomic_t alarm_flag;
+static volatile sig_atomic_t alarm_flag, child_exited;
 static char *argv_bash[] = {"bash", NULL};
-static char *envp_bash[] = {"HOME=/", "TERM=linux", "PATH=/bin:/sbin:/usr/bin:/usr/sbin:.", NULL};
+static char *envp_bash[] = {"HOME=/", "TERM=linux",
+	"PATH=/bin:/sbin:/usr/bin:/usr/sbin:.", NULL};
 
 int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags);
 
@@ -76,6 +77,18 @@ int execvep(const char *path, char *const argv[], char *const envp[])
 		return execve(path, argv, envp);
 }
 
+int set_not_blk(int fd)
+{
+	int oldfl, ret;
+
+	if ((oldfl = fcntl(fd, F_GETFL)) == -1)
+		return -1;
+	oldfl |= O_NONBLOCK;
+	ret = fcntl(fd, F_SETFL, oldfl);
+
+	return ret;	
+}
+
 static int stdredir(int rdfd, int wrfd)
 {
 	int lenr, lenw, lentotal, lenremain;
@@ -109,12 +122,13 @@ static int stdredir(int rdfd, int wrfd)
 
 static void exec_handler(int sig)
 {
-        return;
+	child_exited = 1;
+	return;
 }
 
 static void alarm_handler(int sig)
 {
-        alarm_flag = 1;
+	alarm_flag = 1;
 	return;
 }
 
@@ -122,7 +136,7 @@ static int vps_real_exec(vps_handler *h, envid_t veid, char *root,
 	int exec_mode, char *const argv[], char *const envp[], char *std_in,
 	int timeout)
 {
-	int oldfl, ret, pid, status, option;
+	int ret, pid, status;
 	int in[2], out[2], err[2], st[2];
 	int fl = 0;
 	struct sigaction act;	
@@ -136,14 +150,8 @@ static int vps_real_exec(vps_handler *h, envid_t veid, char *root,
 		return VZ_RESOURCE_ERROR;
 	}
 	/* Set non block mode */
-	if ((oldfl = fcntl(out[0], F_GETFL)) != -1) {
-		oldfl |= O_NONBLOCK;
-		fcntl(out[0], F_SETFL, oldfl);
-	}
-	if ((oldfl = fcntl(err[0], F_GETFL)) != -1) {
-		oldfl |= O_NONBLOCK;
-		fcntl(err[0], F_SETFL, oldfl);
-	}
+	set_not_blk(out[0]);
+	set_not_blk(err[0]);
 	/* Setup alarm handler */
 	if (timeout) {
 		alarm_flag = 0;
@@ -153,6 +161,7 @@ static int vps_real_exec(vps_handler *h, envid_t veid, char *root,
 		sigaction(SIGALRM, &act, NULL);
                 alarm(timeout);
         }
+	child_exited = 0;
         sigemptyset(&act.sa_mask);
         act.sa_flags = SA_NOCLDSTOP;
         act.sa_handler = exec_handler;
@@ -229,8 +238,7 @@ env_err:
 		/* do not set STDIN_FILENO in select() */
 		fl = 4;
 	}
-	option = WNOHANG;
-	while(1) {
+	while(!child_exited) {
 		int n;
 		fd_set rd_set;
 
@@ -238,29 +246,12 @@ env_err:
 			logger(0, 0, "Execution timeout expired");
 			kill(pid, SIGTERM);
 			alarm(0);
-		}
-		ret = waitpid(pid, &status, option);
-		if (ret == pid) {
-			ret = VZ_SYSTEM_ERROR;
-			if (WIFEXITED(status))
-				ret = WEXITSTATUS(status);
-			else if (WIFSIGNALED(status)) {
-				logger(0, 0, "Got signal %d",
-					WTERMSIG(status));
-				if (timeout && alarm_flag)
-					ret = VZ_EXEC_TIMEOUT;
-			}
-			break;
-		} else if (pid == -1 && errno != EINTR) {
-			ret = VZ_SYSTEM_ERROR;
-			logger(0, errno, "Error in waitpid()");
 			break;
 		}
 		if ((fl & 3) == 3) {
 			/* all fd are closed */
-			option = 0;
 			close(in[1]);
-			continue;
+			break;
 		}
 		FD_ZERO(&rd_set);
 		if (!(fl & 4))
@@ -290,7 +281,7 @@ env_err:
 			logger(0, errno, "Error in select()");
 			close(out[0]);
 			close(err[0]);
-			option = 0;
+			break;
 		}
 	}
 	/* Flush fds */
@@ -300,6 +291,24 @@ env_err:
 	if (!(fl & 2)) {
 		while (stdredir(err[0], STDERR_FILENO) == 0);
 	}
+	while ((ret = waitpid(pid, &status, 0)) == -1)
+		if (errno != EINTR)
+			break;
+	if (ret == pid) {
+		ret = VZ_SYSTEM_ERROR;
+		if (WIFEXITED(status))
+			ret = WEXITSTATUS(status);
+		else if (WIFSIGNALED(status)) {
+			logger(0, 0, "Got signal %d",
+				WTERMSIG(status));
+			if (timeout && alarm_flag)
+				ret = VZ_EXEC_TIMEOUT;
+		}
+	} else if (pid == -1 && errno != EINTR) {
+		ret = VZ_SYSTEM_ERROR;
+		logger(0, errno, "Error in waitpid()");
+	}
+
 err:
 	close(st[0]); close(st[1]);
 	close(out[0]); close(out[1]);
@@ -385,3 +394,56 @@ int vps_exec_script(vps_handler *h, envid_t veid, char *root,
 	return ret;
 }
 
+
+int vps_run_script(vps_handler *h, envid_t veid, char *script, vps_param *vps_p)
+{
+	int is_run;
+	int rd_p[2], wr_p[2];
+	int ret, retry;
+	char *argv[2];
+	char *root = vps_p->res.fs.root;
+
+	if (!stat_file(script))	{
+		logger(0, 0, "Script not found: %s", script);
+		return VZ_NOSCRIPT;
+	}
+	if (pipe(rd_p) || pipe(wr_p)) {
+		logger(0, errno, "Unable to create pipe");
+		return VZ_RESOURCE_ERROR;
+	}
+	if (check_var(root, "VE_ROOT is not set"))
+		return VZ_VE_ROOT_NOTSET;
+	if (check_var(vps_p->res.fs.private, "VE_PRIVATE is not set"))
+		return VZ_VE_PRIVATE_NOTSET;
+	if (!stat_file(vps_p->res.fs.private)) {
+		logger(0, 0, "VPS private area %s does not exist",
+			vps_p->res.fs.private);
+                return VZ_FS_NOPRVT;	
+	}
+	if (!(is_run = vps_is_run(h, veid))) {
+		if (!vps_is_mounted(root)) {
+			if ((ret = fsmount(veid, &vps_p->res.fs,
+				&vps_p->res.dq )))
+			{
+				return ret;
+			}
+		}
+		if ((ret = vz_env_create(h, veid, &vps_p->res, rd_p, wr_p,
+			NULL, NULL)))
+		{
+			return ret;
+		}
+	}
+	argv[0] = script;
+	argv[1] = NULL;
+	ret = vps_exec_script(h, veid, root, argv, NULL, script,
+		NULL, 0);
+	if (!is_run) {
+		write(rd_p[1], &ret, sizeof(ret));
+		retry = 0;
+		while (retry++ < 10 && vps_is_run(h, veid))
+			usleep(500000);
+		fsumount(veid, root);
+	}
+	return ret;
+}
